<?php

namespace Drupal\janrain_connect\EventSubscriber;

use Drupal\janrain_connect\Service\JanrainConnectFlowExtractorService;
use Drupal\user\UserDataInterface;
use Drupal\Core\Config\ConfigFactory;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\janrain_connect\Service\JanrainConnectUsers;
use Drupal\janrain_connect_validate\Service\JanrainConnectValidateMessagesMapping;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\janrain_connect\Service\JanrainConnectConnector;
use Drupal\janrain_connect\Constants\JanrainConnectWebServiceConstants;
use Drupal\janrain_connect\Constants\JanrainConnectConstants;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\janrain_connect\Event\JanrainConnectEvents;
use Drupal\janrain_connect\Event\JanrainConnectSubmitEvent;
use Drupal\Core\Session\SessionManagerInterface;
use Symfony\Component\HttpFoundation\Session\Session;
use Drupal\Core\Session\AccountInterface;

/**
 * Subscribe to Janrain Connect Submit.
 */
class JanrainConnectSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * JanrainConnectFlowExtractorService.
   *
   * @var Drupal\janrain_connect\Service\JanrainConnectFlowExtractorService
   */
  private $janrainFlowExtractorService;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $config;

  /**
   * The user data service.
   *
   * @var \Drupal\user\UserDataInterface
   */
  protected $userData;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * JanrainConnectConnector.
   *
   * @var Drupal\janrain_connect\Service\JanrainConnectConnector
   */
  private $janrainConnector;

  /**
   * JanrainConnectUsers.
   *
   * @var Drupal\janrain_connect\Service\JanrainConnectUsers
   */
  private $janrainUsers;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    JanrainConnectConnector $janrain_connector,
    AccountInterface $current_user,
    SessionManagerInterface $session_manager,
    Session $session,
    ModuleHandlerInterface $module_handler,
    JanrainConnectUsers $janrain_users,
    RequestStack $request_stack,
    UserDataInterface $user_data,
    ConfigFactory $config_factory,
    JanrainConnectFlowExtractorService $jainrain_flow_extractor
  ) {
    $this->janrainConnector = $janrain_connector;
    $this->currentUser = $current_user;
    $this->sessionManager = $session_manager;
    $this->session = $session;
    $this->moduleHandler = $module_handler;
    $this->janrainUsers = $janrain_users;
    $this->requestStack = $request_stack;
    $this->userData = $user_data;
    $this->configFactory = $config_factory;
    $this->config = $config_factory->get('janrain_connect.settings');
    $this->janrainFlowExtractor = $jainrain_flow_extractor;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {

    $events[JanrainConnectEvents::EVENT_SUBMIT][] = ['submitRequestToJanrain'];

    return $events;
  }

  /**
   * Submit Registration.
   */
  public function submitRequestToJanrain(JanrainConnectSubmitEvent $event) {
    $data = $event->getData();

    switch ($event->getFormId()) {
      case JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_REGISTRATION:
        $this->sendRegistrationToJanrain($data);
        break;

      case JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_SIGNIN:
        $this->sendLoginToJanrain($data);
        break;

      case JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_EDIT_PROFILE:
        $this->sendProfileUpdateToJanrain($data);
        break;

      case JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_FORGOT_PASSWORD:
        $this->sendForgotPasswordToJanrain($data);
        break;

      case JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_CHANGE_PASSWORD:
        $this->sendChangePasswordToJanrain($data);
        break;
    }
  }

  /**
   * Send registration data to Janrain.
   *
   * @param array $data
   *   User data to create a janrain registration.
   */
  private function sendRegistrationToJanrain(array $data) {

    $result = $this->janrainConnector->register($data, JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_REGISTRATION);

    if (!empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS]) && $result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_SUCCESS) {

      // Message success should be translatable @codingStandardsIgnoreLine.
      drupal_set_message($this->t(JanrainConnectConstants::JANRAIN_CONNECT_REGISTRATION_MESSAGE_SUCCESS), 'status');

      // Persist user in Drupal site.
      $status = $this->janrainUsers->persistUser($data, $result);

      if ($status == SAVED_NEW) {

        // Welcome Message should be translatable @codingStandardsIgnoreLine.
        drupal_set_message($this->t(JanrainConnectConstants::JANRAIN_CONNECT_REGISTRATION_MESSAGE_WELCOME), 'status');

        $this->requestStack->getCurrentRequest()->query->set('destination', '/');

      }
    }

    // If exists janrain_connect validate, use it for get messages.
    if ($this->moduleHandler->moduleExists('janrain_connect_validate')) {

      // Use direct access because is possible unistall the Janrain Connect
      // Validate. @codingStandardsIgnoreLine
      $messages = JanrainConnectValidateMessagesMapping::getMessages($result, JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_REGISTRATION);

    }
    else {
      $messages[] = JanrainConnectConstants::JANRAIN_CONNECT_REGISTRATION_GENERIC_ERROR;
    }

    foreach ($messages as $message) {
      // The message should be translatable @codingStandardsIgnoreLine.
      drupal_set_message($message, 'error');
    }

    return FALSE;
  }

  /**
   * Send login data to Janrain.
   *
   * @param array $data
   *   Required data to allow user login.
   */
  private function sendLoginToJanrain(array $data) {

    $result = $this->janrainConnector->login(
      $data['signInEmailAddress'],
      $data['currentPassword']
    );

    if (empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS])
      || $result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_ERROR) {
      $this->printErrorMessages($result);
    }

    if ($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_SUCCESS) {

      $this->session->set('janrain_connect_access_token', $result['access_token']);

      // Load by e-mail because we don't have the UUID.
      $user = user_load_by_mail($data['signInEmailAddress']);

      // If user exists on janrain but not exists in Drupal, create user.
      if (empty($user)) {

        $data_register['emailAddress'] = $data['signInEmailAddress'];

        $user = $this->janrainUsers->persistUser($data_register, $result, TRUE);

      }

      // User Login.
      user_login_finalize($user);

      // Message success should be translatable @codingStandardsIgnoreLine.
      drupal_set_message('You are successfully logged in', 'status');

      $this->requestStack->getCurrentRequest()->query->set('destination', '/');
    }
  }

  /**
   * Send profile update to Janrain.
   *
   * @param array $data
   *   User data to update a profile in Janrain.
   */
  private function sendProfileUpdateToJanrain(array $data) {
    $accessToken = $this->session->get('janrain_connect_access_token');

    // TODO: Remove this condition after the whole login process
    // is done.
    if (empty($accessToken)) {
      // @codingStandardsIgnoreLine.
      drupal_set_message('You are not logged in', 'warning');
      return;
    }

    $result = $this->janrainConnector->updateUserProfile($accessToken, $data);

    if (empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS])
      || $result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_ERROR) {
      $this->printErrorMessages($result);
    }

    if ($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_SUCCESS) {
      // Message success should be translatable @codingStandardsIgnoreLine.
      drupal_set_message('Profile successfully updated', 'status');
    }
  }

  /**
   * Send Forgot Password to Janrain.
   *
   * @param array $data
   *   User data to create a janrain registration.
   */
  // @codingStandardsIgnoreLine
  private function sendForgotPasswordToJanrain(array $data) {

    $result = $this->janrainConnector->forgotPassword($data['signInEmailAddress'], JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_FORGOT_PASSWORD);

    if (empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS]) || $result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_ERROR) {
      $this->printErrorMessages($result);
    }

    if ($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_SUCCESS) {
      drupal_set_message($this->t("We've sent an email with instructions to create a new password. Your existing password has not been changed."), 'status');
    }
  }

  /**
   * Send Change Password to Janrain.
   *
   * @param array $data
   *   User data to change password.
   */
  private function sendChangePasswordToJanrain(array $data) {

    $newPassword = $data['newPassword'];
    $newPasswordConfirm = $data['newPasswordConfirm'];
    $token = $this->session->get('janrain_connect_access_token');

    $result = $this->janrainConnector->changePassword($newPassword, $newPasswordConfirm, $token, NULL, NULL, JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FORM_CHANGE_PASSWORD);

    if ($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_SUCCESS) {
      // Message of success should be translatable @codingStandardsIgnoreLine.
      drupal_set_message($this->t(JanrainConnectConstants::JANRAIN_CONNECT_CHANGE_PASSWORD_MESSAGE_SUCCESS), 'status');
    }

    if (empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS]) || $result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS] == JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS_ERROR) {
      $this->printErrorMessages($result);
    }
  }

  /**
   * Print error messages.
   */
  private function printErrorMessages($result) {

    if (empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_STATUS])) {
        // 'Unknown error' should be translatable @codingStandardsIgnoreLine.
      drupal_set_message(t(JanrainConstants::JANRAIN_CONNECT_UNKNOWN_ERROR));
    }

    drupal_set_message($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_ERROR_DESCRIPTION], 'error');

    if (!empty($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_INVALID_FIELDS])) {
      foreach ($result[JanrainConnectWebServiceConstants::JANRAIN_CONNECT_INVALID_FIELDS] as $invalid_field) {
        if (empty($invalid_field[0])) {
          continue;
        }

        drupal_set_message($invalid_field[0], 'error');
      }
    }
  }

}
