<?php

namespace Drupal\janrain_connect\Form;

use Drupal\janrain_connect\Constants\JanrainConnectWebServiceConstants;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\janrain_connect\Service\JanrainConnectFormService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\janrain_connect\Event\JanrainConnectEvents;
use Drupal\janrain_connect\Event\JanrainConnectSubmitEvent;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Symfony\Component\HttpFoundation\Session\Session;
use Drupal\Core\Session\AccountInterface;

/**
 * Form for configure messages.
 */
class JanrainConnectForm extends FormBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The event dispatcher service.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Janrain Connect Form Service.
   *
   * @var Drupal\janrain_connect\Service\JanrainConnectFormService
   */
  private $janrainConnectFormService;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactory $config_factory,
    JanrainConnectFormService $janrain_connect_form_service,
    EventDispatcherInterface $event_dispatcher,
    AccountInterface $current_user,
    SessionManagerInterface $session_manager,
    Session $session
  ) {
    $this->configFactory = $config_factory;
    $this->config = $config_factory->get('janrain_connect.settings');
    $this->janrainConnectFormService = $janrain_connect_form_service;
    $this->eventDispatcher = $event_dispatcher;
    $this->currentUser = $current_user;
    $this->sessionManager = $session_manager;
    $this->session = $session;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('janrain_connect.form'),
      $container->get('event_dispatcher'),
      $container->get('current_user'),
      $container->get('session_manager'),
      $container->get('session')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'janrain_connect_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $form_id = '') {

    // We need check if is test and dont call janrain service.
    // @codingStandardsIgnoreLine
    $path = \Drupal::service('path.current')->getPath();
    $path = explode('/', $path);

    // Default value.
    $data_for_tests = FALSE;

    if (!empty(end($path) && end($path) == 'test')) {
      $data_for_tests = TRUE;
    }

    $form_data = $this->janrainConnectFormService->getForm($form_id, $data_for_tests);

    if (empty($form_data)) {
      throw new NotFoundHttpException();
    }

    $form_state->set('form_id', $form_id);

    $fields = $form_data['fields'];

    foreach ($fields as $key => $field) {

      $field_id = $key;

      $label = $field['label'];
      $type = 'textfield';
      $required = FALSE;
      $options = FALSE;

      if (!empty($field['validations']['required'])) {
        $required = $field['validations']['required'];
      }

      $type = $field['type'];
      $field_mapping = $this->config->get('field_mapping');

      if (!empty($field_mapping[$type])) {
        $type = $field_mapping[$type];
      }

      $options = $field['options'];

      $classes = $field['classes'];

      $form[$field_id] = [
        '#type' => $type,
        '#title' => $label,
        '#required' => $required,
      ];

      if ($options) {
        $form[$field_id]['#options'] = $options;
      }

      if ($classes) {
        $form[$field_id]['#attributes']['class'] = $classes;
      }

      if ($type === 'date') {
        $form[$field_id]['#attributes']['type'] = $type;
      }

      if ($field['placeholder']) {
        $form[$field_id]['#attributes']['placeholder'] = $field['placeholder'];
      }

      if ($field['description']) {
        $form[$field_id]['#description'] = $field['description'];
      }
    }

    $submit_name = $this->t('Submit');

    $config_form_data = $this->config->get('config_form_data');

    if (!empty($config_form_data[$form_id]['submit_button'])) {
      // Submit button should be translatable @codingStandardsIgnoreLine.
      $submit_name = $this->t($config_form_data[$form_id]['submit_button']);
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $submit_name,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $form_id = '';
    $data = [];
    $drupal_internal_form_keys = [
      'submit',
      'form_build_id',
      'form_token',
      'form_id',
      'op',
    ];

    if (!empty($form_state->get('form_id'))) {
      $form_id = $form_state->get('form_id');
    }

    if (!empty($form_state->getValues())) {

      $data = $form_state->getValues();

      // Remove Drupal internal form keys.
      foreach ($drupal_internal_form_keys as $key) {
        unset($data[$key]);
      }
    }

    foreach ($data as $key => $current_data) {

      if (empty($form[$key]['#attributes']['class'])) {
        continue;
      }

      $classes = $form[$key]['#attributes']['class'];

      if (!empty($current_data) && in_array('dateselect', $classes)) {

        $field_date_select = $current_data;

        $field_date_select_array = explode("-", $field_date_select);

        $dateselect_day = $field_date_select_array[2];
        $dateselect_month = $field_date_select_array[1];
        $dateselect_year = $field_date_select_array[0];

        $data[$key] = [
          JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FIELD_DATESELECT_DAY => $dateselect_day,
          JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FIELD_DATESELECT_MONTH => $dateselect_month,
          JanrainConnectWebServiceConstants::JANRAIN_CONNECT_FIELD_DATESELECT_YEAR => $dateselect_year,
        ];
      }
    }

    $event = new JanrainConnectSubmitEvent($form_id, $data);

    $this->eventDispatcher->dispatch(JanrainConnectEvents::EVENT_SUBMIT, $event);
  }

}
