<?php

namespace Drupal\janrain_connect\Form;

use Drupal\janrain_connect\Service\JanrainConnectFormService;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\janrain_connect\Service\JanrainConnectFlow;
use Drupal\Core\Config\ConfigFactory;
use Drupal\janrain_connect\Service\JanrainConnectFlowExtractorService;

/**
 * Class for show forms.
 */
class JanrainConnectShowForms extends ConfigFormBase {

  /**
   * Janrain Connect FOrm Service.
   *
   * @var Drupal\janrain_connect\Service\JanrainConnectFormService
   */
  private $janrainConnectFormService;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;
  protected $configFactory;
  private $janrainConnectFlowExtractor;
  private $janrainConnectFlow;

  /**
   * The service container.
   *
   * @var \Symfony\Component\DependencyInjection\ContainerInterface
   */
  protected $container;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    LanguageManagerInterface $language_manager,
    JanrainConnectFlow $janrain_connect_flow,
    JanrainConnectFormService $janrain_connect_form_service,
    ConfigFactory $config_factory,
    ContainerInterface $container,
    JanrainConnectFlowExtractorService $jainrain_flow_extractor
  ) {
    $this->languageManager = $language_manager;
    $this->janrainConnectFlow = $janrain_connect_flow;
    $this->janrainConnectFormService = $janrain_connect_form_service;
    $this->configFactory = $config_factory;
    $this->container = $container;
    $this->janrainConnectFlowExtractor = $jainrain_flow_extractor;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
        $container->get('language_manager'),
        $container->get('janrain_connect.flow'),
        $container->get('janrain_connect.form'),
        $container->get('config.factory'),
        $container->get('service_container'),
        $container->get('janrain_connect.flow_extractor')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'janrain_connect_sync';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'janrain_connect.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = $this->config('janrain_connect.settings');
    $application_id = $config->get('application_id');
    $flowjs_url = $config->get('flowjs_url');

    $forms_to_show = $this->janrainConnectFlowExtractor->getFormsData();

    if (!$forms_to_show) {
      drupal_set_message($this->t('No Janrain Forms were found in the flow. Did you perform Janrain Sync?'), 'warning');
      return [];
    }

    // Check configurations.
    if (empty($application_id) || empty($flowjs_url)) {

      drupal_set_message($this->t('Please fill Janrain settings.'), 'error');

      $url_settings = Url::fromRoute('janrain_connect.settings')->toString();

      return new RedirectResponse($url_settings);
    }

    $form['forms_as_block'] = [
      '#type' => 'details',
      '#title' => 'Forms as Block',
      '#open' => TRUE,
    ];

    // Default values.
    $options_forms = [];

    foreach ($forms_to_show as $key => $form_to_show) {

      // Replace because user not allow char ".".
      $form_id = str_replace('.', '@DOT@', $key);

      $options_forms[$form_id] = $form_id;
    }

    $forms_as_block = $config->get('forms_as_block');

    if (!$forms_as_block) {
      $forms_as_block = [];
    }

    $form['forms_as_block']['forms_as_block'] = [
      '#type' => 'checkboxes',
      '#options' => $options_forms,
      '#title' => $this->t('Forms to render as block'),
      '#default_value' => $forms_as_block,
    ];

    $forms_to_show_keys = array_keys($forms_to_show);

    foreach ($forms_to_show_keys as $key) {

      $form_data = $this->janrainConnectFormService->getForm($key);

      if (!$form_data) {
        break;
      }

      $form_id = $form_data['id'];
      $fields = !empty($form_data['fields']) ? $form_data['fields'] : NULL;

      $host = Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString();

      $link = $host . 'janrain/form/' . $form_id;

      $output = '<h3>' . $this->t('ID') . '</h3>';
      $output .= '<p>' . $form_id . '</p>';

      $output .= '<h3>' . $this->t('Link') . '</h3>';
      $output .= '<p><a target="blank" href="' . $link . '">' . $link . '</a></p>';

      if (!empty($fields)) {

        $output .= '<h3>' . $this->t('Fields') . '</h3>';

        $output .= '<ul>';

        foreach ($fields as $field) {
          $label = $field['label'];
          $output .= '<li>' . $label . '</li>';
        }

        $output .= '</ul>';
      }

      $form[$form_id] = [
        '#type' => 'details',
        '#title' => $form_id,
        '#open' => TRUE,
      ];

      $form[$form_id]['details'] = [
        '#type' => 'markup',
        '#markup' => $output,
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    parent::submitForm($form, $form_state);

    $config = $this->config('janrain_connect.settings');

    $forms_as_block = $form_state->getValue('forms_as_block');

    $config->set('forms_as_block', $forms_as_block);

    $config->save();
  }

}
